/***************************************************************************
                          kscopeconfig.h  -  description
                             -------------------
    begin                : Fri Jun 13 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSCOPECONFIG_H
#define KSCOPECONFIG_H

#include <qobject.h>
#include <qstringlist.h>
#include <qcolor.h>
#include <kdockwidget.h>

typedef QValueList<int> SPLIT_SIZES;

/**
 * Loads and stores global configuration parameters.
 * @author Elad Lahav
 */

class KScopeConfig : public QObject
{
	Q_OBJECT
	
public:
	enum ConfWidget { FileList = 0, TagList, QueryList };
	enum ColorType { Background = 0, Foreground = 1 };
	enum CtagSort { NameAsc = 0, NameDes, LineAsc, LineDes, TypeAsc,
		TypeDes };
	
	KScopeConfig();
	~KScopeConfig();

	void load();
	void loadDefault();
	void loadWorkspace(KDockMainWindow*);
	void store();
	void storeWorkspace(KDockMainWindow*);
	bool isFirstTime();
	
	const QString& getCscopePath() const;
	void setCscopePath(const QString&);
	bool useVerboseCscope() const;
	void setVerboseCscope(bool);
	const QString& getCtagsPath() const;
	void setCtagsPath(const QString& sPath);
	const QStringList& getRecentProjects() const;
	void addRecentProject(const QString&);
	void removeRecentProject(const QString&);
	const SPLIT_SIZES& getEditorSizes() const;
	void setEditorSizes(const SPLIT_SIZES&);
	const QColor& getColor(ConfWidget, ColorType) const;
	void setColor(ConfWidget, ColorType, const QColor&);
	const QFont& getFont(ConfWidget) const;
	void setFont(ConfWidget, const QFont&);
	uint getMaxPosHistory();
	CtagSort getCtagSortOrder();
	void setCtagSortOrder(CtagSort);
	bool getReadOnlyMode();
	void setReadOnlyMode(bool);
	bool getLoadLastProj();
	void setLoadLastProj(bool);
	bool getAutoTagHl();
	void setAutoTagHl(bool);
	bool getUseBriefQueryCaptions();
	void setUseBriefQueryCaptions(bool);
	bool getShowQueryButtons();
	void setShowQueryButtons(bool);
	const QString& getExtEditor();
	void setExtEditor(const QString&);
	bool useExtEditor();
	
signals:
	void colorChanged(ConfWidget, ColorType);
	void fontChanged(ConfWidget);

private:
	QStringList m_slProjects;

	/** Defines the list of all configurable parameters in KScope.
		The use of a structure helps define default values (@see s_cpDef) */
	struct ConfParams {
		QString sCscopePath;
		bool bVerboseCscope;
		QString sCtagsPath;
		SPLIT_SIZES siEditor;
		QColor clrFileListBack;
		QColor clrFileListFore;
		QColor clrTagListBack;
		QColor clrTagListFore;
		QColor clrQueryListBack;
		QColor clrQueryListFore;
		QFont fontFileList;
		QFont fontTagList;
		QFont fontQueryList;
		uint nMaxPosHistory;
		CtagSort ctagSortOrder;
		bool bReadOnlyMode;
		bool bLoadLastProj;
		bool bAutoTagHl;
		bool bBriefQueryCaptions;
		bool bShowQueryButtons;
		QString sExtEditor;
	};

	/** The current configuration parameters */
	ConfParams m_cp;

	/** Holds default values for the configuration parameters */
	static ConfParams s_cpDef;
	
	/** Write font preferences only if modified by the user (keep default
		setting otherwise) */
	bool m_bFontsChanged;
};

KScopeConfig& Config();

#endif
