/***************************************************************************
                          positionhistory.h
                             -------------------
    begin                : Sat April 3 2004
    copyright            : (C) 2004 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/
 
#ifndef POSITIONHISTORY_H
#define POSITIONHISTORY_H

#include <qstring.h>
#include <qvaluelist.h>
#include <qlistview.h>

class PositionListTarget
{
public:
	virtual void addItem(const QString& sFilePath, uint nLine, int nIndex, 
		bool bCurrent) = 0;
};

/**
 * Keeps track of cursor positions set by Cscope or Ctags queries.
 * @author Elad Lahav
 */
class PositionHistory
{
public:
    PositionHistory();
    ~PositionHistory();
	
	void accept(bool);
	void add(const QString&, uint);
	bool current(QString&, uint&);
	bool next(QString&, uint&);
	bool prev(QString&, uint&);
	void moveTo(int);
	void clear();
	bool fillList(PositionListTarget*);
	
private:
	
	/** Must be set to true for new entries to be added to the history. */
	bool m_bAccept;
	
	class FilePosition
	{
	public:
		FilePosition() {}
		FilePosition(const QString& sFilePath, uint nLine) : 
			m_sFilePath(sFilePath),
			m_nLine(nLine) {}
		
		QString m_sFilePath;
		uint m_nLine;
	};

	/** A list of FilePosition items which serves as the history stack. */
	QValueList<FilePosition> m_list;
	
	/** Points to the current item in the hostory list. */
	QValueList<FilePosition>::Iterator m_itr;
};

PositionHistory& PosHistory();

#endif
