/***************************************************************************
                          positionhistorydlg.cpp
                             -------------------
    begin                : Sun April 4 2004
    copyright            : (C) 2004 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

#include <qlistview.h>
#include <qpushbutton.h>
#include "positionhistorydlg.h"

/**
 * Class constructor.
 * @param	pParent	The parent widget
 * @param	szName	The widget's name
 */
PositionHistoryDlg::PositionHistoryDlg(QWidget* pParent, const char* szName) : 
	PositionHistoryLayout(pParent, szName, true, 0),
	m_pItem(NULL)
{
	// Turn sorting off - diaply items the same order they were inserted
	m_pPosList->setSorting(-1);
	
	// Clicking the "Go" makes KScope jump to the position given by the 
	// selected item
	connect(m_pGoButton, SIGNAL(clicked()), this, SLOT(accept()));

	// Double-clicking an item makes KScope jump to the position given by this
	// item
	connect(m_pPosList, 
		SIGNAL(doubleClicked(QListViewItem*, const QPoint &, int)),
		this,
		SLOT(slotItemDoubleClicked(QListViewItem*, const QPoint&, int)));
	
	// Clicking the "Close" button simply closes the dialog, without taking 
	// any other action
	connect(m_pCloseButton, SIGNAL(clicked()), this, SLOT(reject()));
	
	PosHistory().fillList(this);
}

/**
 * Class destructor.
 */
PositionHistoryDlg::~PositionHistoryDlg()
{
}

/**
 * Called when an item is selected by either clicking the "Go" button or
 * double-clicking the item.
 * Moves the position history iterator to the index of the item in the list,
 * and closes the dialog.
 */
void PositionHistoryDlg::accept()
{
	ListViewItem* pItem;
	
	// Find the selected item, if any
	if ((pItem = (ListViewItem*)m_pPosList->selectedItem()) == 0) {
		QDialog::reject();
		return;
	}
	
	// Mark the selected item in the position history list
	PosHistory().moveTo(pItem->m_nIndex);
	
	// Close the dialog
	QDialog::accept();
}

/**
 * Called by PositionHistory::fillList() to add each item in the position
 * history list.
 * Creates a ListViewItem for each item, and marks the current position.
 */
void PositionHistoryDlg::addItem(const QString& sFilePath, uint nLine, 
	int nIndex, bool bCurrent)
{
	// Create a new list item
	m_pItem = new ListViewItem(m_pPosList, m_pItem, sFilePath, 
		QString::number(nLine));
	m_pItem->m_nIndex = nIndex;
	
	// Mark the current position
	if (bCurrent)
		m_pPosList->setSelected(m_pItem, true);
}

/**
 * Close the dialog when an item is double-clicked.
 */
void PositionHistoryDlg::slotItemDoubleClicked(QListViewItem*, const QPoint&, 
	int)
{
	accept();
}

#include "positionhistorydlg.moc"
